#ifndef __UTM_URFACLIENT_H__
#define __UTM_URFACLIENT_H__

/**
 * NetUP UTM5 urfaclient library header file
 * Copyright (c) NetUP Inc., 2010
 */

#define DECLSPEC

#ifdef __cplusplus
extern "C" {
#endif

/**
 * User types for <code>urfa_open_session</code>
 */

//Administrator login
#define UT_ADMIN    -1
//User login
#define UT_USER     -2


/**
 * URFA data types
 */

//Unknown
#define URFA_NONE   0
//32-bit integer
#define URFA_INT    1
//64-bit integer
#define URFA_LONG   2
//String
#define URFA_STRING 3
//Double
#define URFA_DOUBLE 4
//Array
#define URFA_ARRAY  5

/**
 * URFA session context structure
 */
typedef struct {
    // pointer to implementation
    void * impl;
} URFA_SESSION;

/**
 * URFA data structure
 */
typedef struct {
    //URFA variable name
    const char * name;

    //URFA variable type
    int type;

    //next variable in the array
    void * next;

    //URFA variable value
    union {
        // string
        char * str_val;
        // 32-bit integer
        int int_val;
        // 64-bit integer
        long long long_val;
        // double
        double double_val;
        // URFA_DATA
        void * data_val;
    };

} URFA_DATA;

/**
 * URFA error codes list
 */
enum {
    // no error
    URFA_ERROR_NONE = 0,
    // failed to init URFAClient library
    URFA_ERROR_INIT = 1,
    // unable to connect to UTM5 core
    URFA_ERROR_CONNECT = 2,
    // unable to login
    URFA_ERROR_AUTH = 3,
    // URFA call error
    URFA_ERROR_CALL = 4,
    // URFA function not found
    URFA_ERROR_NO_SUCH_FUNC = 5,
    // library already initialized
    URFA_ERROR_INITIALIZED = 6,
    // library not initialized
    URFA_ERROR_NOT_INITIALIZED = 7,
    // bad argument for this function
    URFA_ERROR_BAD_ARGUMENT = 8,
    // unable to set URFA data
    URFA_ERROR_SET = 9,
    // unable to get URFA data
    URFA_ERROR_GET = 10,
} urfa_error_codes;

/**
 * Function <code>urfa_init_lib</code> setups the libraty and must be called before using other library functions
 * @param api_xml_path Path to api.xml file
 * @return 0 on error, and urfa error state set to approptiate value
 * @see urfa_get_error
 */
DECLSPEC int urfa_init_lib(const char * api_xml_path);

/**
 * Function <code>urfa_open_session</code> opens the connection to UTM5 server core
 * @param host UTM5 core host, for example "127.0.0.1"
 * @param port UTM5 core port, for example 11758
 * @param login user or administrator login
 * @param password user or administrator password
 * @param user_type <code>UT_USER</code> for user session or <code>UT_ADMIN</code> for administrator session
 * @return pointer to new URFA session context on success or NULL on error
 * @see urfa_get_error
 */
DECLSPEC URFA_SESSION * urfa_open_session(const char * host, int port, const char * login, const char * password, int user_type);

/**
 * Function <code>urfa_close_session</code> closes connection and frees session context allocated by <code>urfa_open_session</code>
 * @param session pointer to valid URFA session context allocated by <code>urfa_open_session</code>
 * @return none
 * @see urfa_open_session
 */
DECLSPEC void urfa_close_session(URFA_SESSION * session);

/**
 * Function <code>urfa_set_input_data<code> sets input data for URFA call
 * @param session pointer to valid URFA session context allocated by <code>urfa_open_session</code>
 * @param data pointer to URFA data structure allocated by <code>urfa_new_data</code> and properly initialized by user
 * @return 0 on error, and urfa error state set to approptiate value
 * @see urfa_get_error
 * @see api.xml file
 */
DECLSPEC int urfa_set_input_data(URFA_SESSION * session, URFA_DATA * data);

/**
 * Function <code>urfa_get_output_data<code> used to obtain results of URFA call for URFA session cotext.
 * @param session pointer to valid URFA session context allocated by <code>urfa_open_session</code>
 * @param name name of wanted URFA data
 * @param data pointer to URFA data structure allocated by <code>urfa_new_data</code> used to store result
 * @return 0 on error, and urfa error state set to approptiate value
 * @see urfa_get_error
 * @see api.xml file
 */
DECLSPEC int urfa_get_output_data(URFA_SESSION * session, const char * name, URFA_DATA * data);

/**
 * Function <code>urfa_call_function</code> invokes the liburfa function
 * @param session pointer to valid URFA session context allocated by <code>urfa_open_session</code>
 * @param function name of URFA function
 * @return 0 on error, and urfa error state set to approptiate value
 * @see urfa_get_error
 * @see api.xml file
 */
DECLSPEC int urfa_call_function(URFA_SESSION * session, const char * function);

/**
 * Function <code>urfa_enable_debug</code>
 * @param debug non-zero value  to enable debug messages, disable otherwise
 * @return always 1
 */
DECLSPEC int urfa_enable_debug(int debug);

/**
 * Function <code>urfa_get_error</code> returns error code of previous operation
 * @return integer error code
 * @see urfa_error_codes union
 */
DECLSPEC int urfa_get_error();

/**
 * Function <code>urfa_error_string</code> returns string representation of URFA error code
 * @param error URFA error code
 * @return pointer to constant string representation of error
 * @see urfa_get_error
 */
DECLSPEC const char * urfa_error_string(int error);

/**
 * Function <code>urfa_new_data</code> allocates new URFA data structure
 * @return pointer to allocated URFA data structure
 * @see urfa_free_data
 */
DECLSPEC URFA_DATA * urfa_new_data();

/**
 * Function <code>urfa_free_data</code> frees URFA data structure allocated by <code>urfa_new_data</code>
 * @param data pointer to valid URFA data allocated by <code>urfa_new_data</code>
 * @see urfa_new_data
 */
DECLSPEC void urfa_free_data(URFA_DATA * data);

#ifdef __cplusplus
}
#endif

#endif // __UTM_URFACLIENT_H__
